﻿using System;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace ChrisTec.WindowsMobile
{
  public enum InputContextClass
  {
    Default,
    Email,
    Url,
    Phone,
    Name,
    PhoneAndEmail
  }

  public static class InputContext
  {
    public static void SetAutoSuggest(Control window, bool enabled)
    {
      SHSetInputContext(window.Handle, SHIC_FEATURE.AUTOSUGGEST, ref enabled);
    }

    public static bool GetAutoSuggest(Control window)
    {
      bool enabled = false;
      uint size = 4;
      SHGetInputContext(window.Handle, SHIC_FEATURE.AUTOSUGGEST, ref enabled, ref size);
      return enabled;
    }

    public static void SetAutoCorrect(Control window, bool enabled)
    {
      SHSetInputContext(window.Handle, SHIC_FEATURE.AUTOCORRECT, ref enabled);
    }

    public static bool GetAutoCorrect(Control window)
    {
      bool enabled = false;
      uint size = 4;
      SHGetInputContext(window.Handle, SHIC_FEATURE.AUTOCORRECT, ref enabled, ref size);
      return enabled;
    }

    public static void SetHaveTrailer(Control window, bool enabled)
    {
      SHSetInputContext(window.Handle, SHIC_FEATURE.HAVETRAILER, ref enabled);
    }

    public static bool GetHaveTrailer(Control window)
    {
      bool enabled = false;
      uint size = 4;
      SHGetInputContext(window.Handle, SHIC_FEATURE.HAVETRAILER, ref enabled, ref size);
      return enabled;
    }

    public static void SetClass(Control window, InputContextClass klass)
    {
      SHSetInputContext(window.Handle, SHIC_FEATURE.CLASS, SHIC_CLASSES[(int)klass]);
    }

    public static InputContextClass GetClass(Control window)
    {
      // Query the OS for the current InputContext class
      StringBuilder sb = new StringBuilder(64);
      uint size = (uint)sb.Capacity;      
      SHGetInputContext(window.Handle, SHIC_FEATURE.CLASS, sb, ref size);
      
      // Convert the string returned by SHGetInputContext into
      // an InputContextClass enumeration value
      int index = Array.IndexOf(SHIC_CLASSES, sb.ToString());
      if (index == -1)
        return InputContextClass.Default;
      else
        return (InputContextClass)index;
    }

    #region Platform Invokes 

    [DllImport("aygshell.dll")]
    private static extern int SHSetInputContext(IntPtr hwnd, SHIC_FEATURE dwFeature, [MarshalAs(UnmanagedType.Bool)]ref bool lpValue);

    [DllImport("aygshell.dll")]
    private static extern int SHSetInputContext(IntPtr hwnd, SHIC_FEATURE dwFeature, string lpValue);

    [DllImport("aygshell.dll")]
    private static extern int SHGetInputContext(IntPtr hwnd, SHIC_FEATURE dwFeature, [MarshalAs(UnmanagedType.Bool)]ref bool lpValue, ref uint lpdwSize);

    [DllImport("aygshell.dll")]
    private static extern int SHGetInputContext(IntPtr hwnd, SHIC_FEATURE dwFeature, StringBuilder lpValue, ref uint lpdwSize);

    private enum SHIC_FEATURE : uint
    {
      RESTOREDEFAULT = 0,
      AUTOCORRECT = 1,
      AUTOSUGGEST = 2,
      HAVETRAILER = 3,
      CLASS = 4
    };

    // These strings must be kept in sync with the InputContextClass enum
    private static readonly string[] SHIC_CLASSES = new string[]{"", "email", "url", "phone", "name", "phoneAndEmail"};

    #endregion
  }
}
