using System;
using System.Windows.Forms;
using Microsoft.WindowsMobile.Status;
using Microsoft.WindowsMobile.PocketOutlook;
using ChrisTec.WindowsMobile;

namespace SendSMS
{
  public partial class MainForm : Form
  {
    public MainForm()
    {
      InitializeComponent();

      txtMessage.Text = "";

      // Setup the input context for the phone number text box
      // so that it will provide auto suggestions of phone numbers
      // associated with outlook/SIM contacts.
      InputContext.SetClass(txtPhoneNumber, InputContextClass.Phone);
      InputContext.SetHaveTrailer(txtPhoneNumber, false);
    }

    private void txtMessage_TextChanged(object sender, EventArgs e)
    {
      // Determine the maximum number of characters allowed before the
      // message requires an additional SMS message to transmit.
      //
      // The maxmimum single text message size is 160 7-bit characters.
      // Larger content can be sent using multiple SMS messages where
      // each message starts with a user data header (UDH) containing
      // segmentation information. This header reduces the payload size
      // of each SMS to 153 characters.
      int maxLength;
      if (txtMessage.Text.Length <= 160)
        // Fits in one message (with max 160 chars)
        maxLength = 160;
      else
        // Requires two or more messages (with max 153 chars per message)
        maxLength = ((txtMessage.Text.Length + 152) / 153) * 153;

      // Update the label
      string messageCount;
      if (maxLength / 153 == 1)
        messageCount = "1 message";
      else
        messageCount = String.Format("{0} messages", maxLength / 153);

      lblMessageSize.Text = String.Format("({0}) {1} / {2}",
        messageCount,
        txtMessage.Text.Length,
        maxLength);
    }

    private SmsMessage CreateSmsMessage()
    {
      SmsMessage message = new SmsMessage();

      message.To.Add(new Recipient(txtPhoneNumber.Text));
      message.Body = txtMessage.Text;
      if (chkIncludeBatteryStatus.Checked)
      {
        message.Body += String.Format(" Battery Status: {0}", SystemState.PowerBatteryStrength);
      }
      
      return message;
    }

    private void mnuSend_Click(object sender, EventArgs e)
    {
      // Send the message without bothering the user
      SmsMessage message = CreateSmsMessage();
      message.Send();
    }

    private void mnuDraft_Click(object sender, EventArgs e)
    {
      // Display the message in the Messaging application
      // to give the user a chance to edit it or save it
      // to the drafts folder etc.
      SmsMessage message = CreateSmsMessage();
      MessagingApplication.DisplayComposeForm(message);
    }
  }
}